Package: BIGDAWG
Type: Package
Title: Case-Cotrol Analysis of Multi-Allelic Loci
Version: 3.1.0
Date: 2026-04-29
Authors@R: c(person("Derek","Pappas",role = "aut",email = "djpappas75@gmail.com"),
               person("Livia","Tran",role = "aut",email = "Livia.Tran@ucsf.edu"),
               person("Steve","Mack",role = c("aut", "cre"),email = "Steven.Mack@ucsf.edu"),
               person("Jill","Hollenbach",role = "aut",email = "Jill.Hollenbach@ucsf.edu"))
Maintainer: Steve Mack <Steven.Mack@ucsf.edu>
URL: https://github.com/IgDAWG/BIGDAWG
BugReports: https://github.com/IgDAWG/BIGDAWG/issues
Description: Data sets and functions for chi-squared Hardy-Weinberg and case-control association tests of highly polymorphic genetic data [e.g., human leukocyte antigen (HLA) data]. Performs association tests at multiple levels of polymorphism (haplotype, locus and HLA amino-acids) as described in Pappas DJ, Marin W, Hollenbach JA, Mack SJ (2016) <doi:10.1016/j.humimm.2015.12.006>. Combines rare variants to a common class to account for sparse cells in tables as described by Hollenbach JA, Mack SJ, Thomson G, Gourraud PA (2012) <doi:10.1007/978-1-61779-842-9_14>.
License: GPL (>= 3)
Depends: R (>= 4.0.0)
Imports: XML, haplo.stats, parallel
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-04-30 05:15:50 UTC; sjmack
Author: Derek Pappas [aut],
  Livia Tran [aut],
  Steve Mack [aut, cre],
  Jill Hollenbach [aut]
Repository: CRAN
Date/Publication: 2026-04-30 08:30:45 UTC
Built: R 4.6.0; ; 2026-04-30 11:23:51 UTC; unix
