exportPattern("^[[:alpha:]]+")
importFrom("parallel","mclapply")
importFrom("graphics", "axis", "mtext", "title")
importFrom("stats", "chisq.test", "fisher.test", "na.omit", "pchisq", "qnorm", "xtabs")
importFrom("utils", "data", "combn", "download.file", "packageVersion", "read.table", "write.table","write.csv")
