#' Example HLA Dataset
#'
#' A synthetic dataset of HLA genotypes for using bigdawg.
#'
#' @format A data frame with 2000 rows and 14 variables
"HLA_data"

#' Exon protein alignments.
#'
#' Alignment object for use in the amino acid analysis.
#'
#' @format A list where each element is an alignment dataframe for a single locus.
"ExonPtnList"
