% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_functions.R
\name{Check.Data}
\alias{Check.Data}
\title{Check Data Structure}
\usage{
Check.Data(Data, System, Convert)
}
\arguments{
\item{Data}{String Type of output.}

\item{System}{Character Genetic system HLA or KIR}

\item{Convert}{String Direction for conversion.}
}
\description{
Check data structure for successful conversion.
}
\note{
This function is for internal use only.
}
